"""
dnd-character is a Python package for integrating DnD characters
into external applications.
"""


__author__ = "Brianna Rainey"
__copyright__ = "Copyright 2019-2020 Brianna Rainey & Markis Cook"
__credits__ = ["Brianna Rainey (Current Programmer)", "Markis Cook (Original Creator)"]
__license__ = "EPL-2.0"
__version__ = "20.11.01"
__maintainer__ = "Brianna Rainey"


from .character import Character
